import click
from pathlib import Path
import sqlite3
from sqlite_utils import Database
from collections import defaultdict
import hashlib

from .settings import PELICAN_METADATA


def _helper(pelican_metadata, line):
    pelican_metadata_length = len(pelican_metadata)
    pelican_tag = line.split(':', 1)[0].lower()
    if pelican_tag == pelican_metadata:
        return line

@click.command()
@click.version_option()
@click.argument("dbname", nargs=1)
@click.argument("table", nargs=1)
@click.argument("paths", type=click.Path(exists=True), nargs=-1, required=True)
def cli(paths, dbname, table):
    """
    Load pelican markdown files into a SQLite database

    """
    db = Database(dbname)
    docs = []

    for the_file in paths:
        metadata_dictionary = {}
        metadata_dictionary = defaultdict(lambda:None, metadata_dictionary)
        start_line = 0
        with open(the_file) as f:
            lines = f.readlines()
        for line in lines:
            for metadata in PELICAN_METADATA:
                item = _helper(metadata, line)
                if item:
                    start_line += 1
                    metadata_dictionary[item.split(':', 1)[0].lower()] = item.split(':', 1)[1].replace('\n', '')
        if 'save_as' not in metadata_dictionary:
            metadata_dictionary['save_as'] = None
        content = ''.join(lines[start_line:])
        doc = {
            "_id": hashlib.sha1(content.encode("utf8")).hexdigest(),
            "content": content,
            **(metadata_dictionary),
        }
        docs.append(doc)
    columns = list(metadata_dictionary.keys()) + ['_id', 'content']
    db[table].upsert_all(docs, pk="_id")


from setuptools import setup, find_packages
import io
import os

VERSION = "0.1.1"


def get_long_description():
    with io.open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="pelican-to-sqlite",
    description="CLI tool for loading local pelican markdown files into a SQLite database",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Ryan Cheley",
    version=VERSION,
    license="Apache License, Version 2.0",
    packages=find_packages(),
    install_requires=["sqlite-utils", "click"],
    extras_require={"test": ["pytest"]},
    tests_require=["pelican-to-sqlite[test]"],
    setup_requires=["pytest-runner"],
    entry_points="""
        [console_scripts]
        pelican-to-sqlite=pelican_to_sqlite.cli:cli
    """,
    url="https://github.com/ryancheley/pelican-to-sqlite",
    project_urls={
        "Issues": "https://github.com/ryancheley/pelican-to-sqlite/issues",
        "CI": "https://github.com/ryancheley/pelican-to-sqlite/actions",
        "Changelog": "https://github.com/ryancheley/pelican-to-sqlite/releases",
    },
    python_requires=">=3.6",
)


PELICAN_METADATA = [
    'title',
    'date',
    'modified',
    'tags',
    'keywords',
    'category',
    'slug',
    'author',
    'authors',
    'summary',
    'lang',
    'translation',
    'status',
    'template',
    'save_as',
    'url'
]